define(['app', 'uicontrols'], function(App, $AppUI){
	// Popup for configs of internet connection check throught GateD

//====================================================================
// Wrapper for dropdown
//====================================================================
function Dropdown(el, properties){
	if(!(this instanceof Dropdown)) return new Dropdown(el, properties);

	this.el = el[0] || el;
	this.hash = {};

	for(var i = 0, buf = properties.split(';'); i < buf.length; i += 2){
		this.hash[buf[i]] = buf[i + 1];
	}
	this.el.setAttribute('data-list', properties);
	this.el.dataset.list = properties;
}
Dropdown.prototype.set = function(id){
	this.el.setAttribute('data-id', id);
	this.el.dataset.id = id;
	this.el.setAttribute('data-value', this.hash[id]);
	this.el.dataset.value = this.hash[id];
	return this;
};
Dropdown.prototype.get = function(){
	return this.el.dataset.id;
};
Dropdown.prototype.disable = function(undo){
	this.el.classList[undo ? 'remove' : 'add']('__disabled');
};

//====================================================================
// Parser for config line
//====================================================================
var 	OPS = {',': 4, ':': 3, ' ': 2, '(': 1, ')': 1},
		_splitPattern = /(\,|\:|\s|\(|\))/g,
		_spacePattern = /\s*(\(|\)|\:|\,)\s*/g,
		_spaceBlockPattern = /\s+/g;

function splitOnLexems(str){
	return str.replace(_spacePattern, function(match, symbol){
		return symbol;
	}).replace(_spaceBlockPattern, ' ').split(_splitPattern);
}
function Bracket(name, arg){
	this.name = name || 'Bracket';
	this.arg = arg;
}
function optimizeRoot(o){
	if(o instanceof Bracket){
		if(o.arg && (o.arg.id == ' ' || o.arg.id == ':')){
			o.collection = {};

			if(o.arg.id == ' '){
				var ref = o.arg.arg1;

				if(ref && ref.id == ':') o.collection[ref.arg1] = ref.arg2.list || ref.arg2;
				ref = o.arg.arg2;
				if(ref && ref.id == ':') o.collection[ref.arg1] = ref.arg2.list || ref.arg2;
			}else{
				o.collection[o.arg.arg1] = o.arg.arg2;
			}
			delete o.arg;
		}	
	}else{
		o = new Bracket(o);
	}
	return o;
}

function Operator(name, arg1, arg2){
	this.id = name;
	
	if(name == ','){
		this.list = [];
		
		if(Array.isArray(arg1.list)){
			mergeList(this.list, arg1.list);
		}else{
			this.list.push(arg1);
		}
		
		if(Array.isArray(arg2.list)){
			mergeList(this.list, arg2.list);
		}else{
			this.list.push(arg2);
		}
	}else{
		if(arg1) this.arg1 = arg1;
		if(arg2) this.arg2 = arg2;
	}
}

function mergeList(a, b){
	var 	i = b.length;
	
	while(i-- > 0) a.push(b[i]);	
	return a;
}

// ReversePolishNotaion
function evaluate(list){
	var 	symbol, op, prevOp, prevSymbol,
			i = 0,
			outPos = 0,
			stackPos = 0,
			len = list.length,
			out = Array(len),
			stack = Array(len);

	for(; i < len; i++){
		if(!list[i]) continue;

		if(!OPS[list[i]]){
			out[outPos] = list[i];
			outPos++;
		}else{
			if(list[i] == ')'){
				while(stackPos-- > 0){
					prevOp = stack[stackPos];
					stack[stackPos] = null;

					if(prevOp instanceof Bracket){		
						// Attention: If bracket without id, would be better to not set item at list
						out[outPos++] = prevOp;		
						prevOp = stack[stackPos-1];

						if(OPS[prevOp] && OPS[prevOp] > OPS[')']){ // so don't up =|,|;
							out[outPos++] = prevOp;
							stack[stackPos--] = undefined;
						}
						break;
					}else{
						// TODO create Operator object
						out[outPos++] = prevOp;
					}
				}
			}else if(list[i] == '('){
				var 	lastOut = out[outPos - 1],
						id;

				if(prevSymbol && !OPS[prevSymbol] && lastOut == prevSymbol){
					id = prevSymbol;
					out[--outPos] = null;
				}
				
				stack[stackPos] = new Bracket(id);
				stackPos++;
			}else{
				prevOp = stack[stackPos-1];

				while(stackPos-- > 0){
					prevOp = stack[stackPos];

					if(OPS[prevOp] >= OPS[list[i]]){
						stack[stackPos + 1] = null;
						if(!OPS[out[outPos-2]] && !OPS[out[outPos-1]] && !(out[outPos-2] instanceof Bracket) && !(out[outPos-1] instanceof Bracket)){
							var op = new Operator(prevOp, out[outPos-2], out[outPos-1]);
							outPos -= 2;
							out[outPos++] = op;
						}else{
							out[outPos++] = prevOp;
						}
					}else{
						break;
					}
				}
				stackPos++;
				stack[stackPos++] = list[i];
			}
		}
		prevSymbol = list[i];
	}
	
	while(stackPos-- > 0){
		out[outPos++] = stack[stackPos];
		stack[stackPos] = null;
	}
	out.length = outPos; // correct only for that kind of tasks
	return out;
}

// Предполагается, что в конце списка будут операторы в отсортированным по приоритету параметрами
function buildAST(ops){
	var last = ops.pop();
	
	if(last instanceof Bracket){
		if(last.name){ // it is function
			last.arg = buildAST(ops);
		} // else usual brackets
	}else if(last == ' ' || last == ',' || last == ':'){
		last = new Operator(last, ops.shift(), buildAST(ops));
	}
	return last;
}
function capitalizeFirst(s){
	return s.charAt(0).toUpperCase() + s.slice(1);
}
//====================================================================
// Popup
//====================================================================
	return function(property, initValue, onSibmit){
		var 	ISOUTPUT = property == 'OutputDivert',
				MENU_LIST_TOKENS = ISOUTPUT ? 'output_divert_modes' : 'input_divert_modes';
		var		_ValueModel = optimizeRoot(buildAST(evaluate(splitOnLexems(initValue[0]))));

		new $AppUI.BasePopupView({
			class: 'dec_wt-popup',
			content: 
			'<form class="dwc-form" data-co="form">' +
				'<button class="dwc_popup-close" type="reset"></button>' +
				'<a href="javascript:void(0)" class="dwc_dropdown dwc_title-dropdown" data-co="select" data-component="dropdown"></a>' + 
				'<p data-co="description" class="dwc_settings_description"></p>' +
				'<div class="dwc_settings_row" data-co="interface-row">' +
					'<div class="dwc_settings_cell"><h4 class="dwc_settings_title">' + App.helpers._('divert_property_interface') + '</h4>' + App.helpers._('divert_property_interface_about') + '</div>' +
					'<div class="dwc_settings_cell"><input type="text" class="dwc-form_base-input" data-co="interface-input" required/></div>' +
				'</div>' +
				'<div class="dwc_settings_row" data-co="protected-row">' +
					'<div class="dwc_settings_cell"><h4 class="dwc_settings_title">' + App.helpers._('divert_property_protected') + '</h4>' + App.helpers._('divert_property_protected_about') + '</div>' +
					'<div class="dwc_settings_cell"><input type="text" class="dwc-form_base-input" data-co="protected-input" required/></div>' +
				'</div>' +
				'<div class="dwc_settings_row" data-co="port-row">' +
					'<div class="dwc_settings_cell"><h4 class="dwc_settings_title">' + App.helpers._('divert_property_port') + '</h4>' + App.helpers._('divert_property_port_about') + '</div>' +
					'<div class="dwc_settings_cell"><input type="text" class="dwc-form_base-input" data-co="port-input" required pattern="^\\d+$" title="' + App.helpers._('port_number_format') + '"/></div>' +
				'</div>' +
				'<div class="dwc_settings_row" data-co="mark-row">' +
					'<div class="dwc_settings_cell"><h4 class="dwc_settings_title">' + App.helpers._('divert_property_mark') + '</h4>' + App.helpers._('divert_property_mark_about') + '</div>' +
					'<div class="dwc_settings_cell"><input type="text" class="dwc-form_base-input" data-co="mark-input" required /></div>' +
				'</div>' +
				'<div class="dwc-form_btn-wrap __left">' + 
					'<button class="dwc_btn __green" type="submit">' + App.helpers._('save_btn') + '</button>' +
					'<button class="dwc_btn" data-co="cancel" type="reset">' + App.helpers._('Cancel') +'</button>' +
				'</div>' +
			'</form>' +
			'',
			onopen: function(view){
				view._select = new Dropdown(view.controls.select[0], App.helpers._(MENU_LIST_TOKENS));
				view._select.set(capitalizeFirst(_ValueModel.name)); // TODO configure (Parse value)
				view.controls.select.trigger('dropdownselected');
			},
			onclose: function(view){
				_ValueModel = null;
			},
			_events: {
				'form submit': function(e){
					e.preventDefault();

					var		selected = this._select.get(),
							settingLine = selected,
							buf,
							properties = [];

					switch(selected){
						case 'Auto': 
						case 'Proxy': 
							if(!ISOUTPUT){
								buf = this.controls.interfaceInput.val();
								buf && properties.push('interface:' + buf);
								buf = this.controls.protectedInput.val().replace(/\s+/g, '');	
								buf && properties.push('protected:' + buf);
							}
							break;
						// case 'Manual': 
						// 	buf = this.controls.portInput.val();
						// 	buf && properties.push('port:' + buf);
						// 	if(!ISOUTPUT){
						// 		buf = this.controls.markInput.val();
						// 		buf && properties.push('mark:' + buf);
						// 	}
						// 	break;
					}

					if(properties.length > 0){
						settingLine += '(' + properties.join(' ') + ')';
					}

					onSibmit(settingLine);
					this.close();
				},	
				'form reset': function(e){
					e.preventDefault();
					this.close();
				},
				'select dropdownselected': function(){
					var 	selected = this._select.get(),
							descriptionId;

					switch(selected){
						case 'Off': 
							descriptionId = 'divert_mode_off'; 
							this.controls.interfaceRow.hide();
							this.controls.protectedRow.hide();
							this.controls.portRow.hide();
							this.controls.markRow.hide();

							this.controls.interfaceInput.prop('disabled', true);
							this.controls.protectedInput.prop('disabled', true);
							this.controls.portInput.prop('disabled', true);
							this.controls.markInput.prop('disabled', true);
							break;
						case 'Auto': 
							descriptionId = 'divert_mode_auto'; 
							this.controls.interfaceRow[ISOUTPUT ? 'hide' : 'show']();
							this.controls.protectedRow[ISOUTPUT ? 'hide' : 'show']();
							this.controls.portRow.hide();
							this.controls.markRow.hide();

							this.controls.interfaceInput.prop('disabled', ISOUTPUT);
							this.controls.protectedInput.prop('disabled', ISOUTPUT);
							this.controls.portInput.prop('disabled', true);
							this.controls.markInput.prop('disabled', true);
							break;
						case 'Proxy': 
							descriptionId = 'divert_mode_proxy'; 
							this.controls.interfaceRow.show();
							this.controls.protectedRow.show();
							this.controls.portRow.hide();
							this.controls.markRow.hide();

							this.controls.interfaceInput.prop('disabled', false);
							this.controls.protectedInput.prop('disabled', false);
							this.controls.portInput.prop('disabled', true);
							this.controls.markInput.prop('disabled', true);
							break;
						// case 'Manual': 
						// 	descriptionId = 'divert_mode_manual'; 
						// 	this.controls.interfaceRow.hide();
						// 	this.controls.protectedRow.hide();
						// 	this.controls.portRow.show();
						// 	this.controls.markRow[ISOUTPUT ? 'hide' : 'show']();

						// 	this.controls.interfaceInput.prop('disabled', true);
						// 	this.controls.protectedInput.prop('disabled', true);
						// 	this.controls.portInput.prop('disabled', false);
						// 	this.controls.markInput.prop('disabled', ISOUTPUT);
						// 	break;
					}
					this.controls.description.text(App.helpers._(descriptionId));

					if(_ValueModel.collection && capitalizeFirst(_ValueModel.name) == selected){
						this.controls.portInput.val(_ValueModel.collection.port || '');
						this.controls.markInput.val(_ValueModel.collection.mark || '');
						this.controls.interfaceInput.val(_ValueModel.collection.interface || '');
						var protected = _ValueModel.collection.protected;
						this.controls.protectedInput.val(protected ? (Array.isArray(protected) ? protected.join(', ') : protected) : '');	
					}else{
						this.controls.portInput.val('');
						this.controls.markInput.val('');
						this.controls.interfaceInput.val('');
						this.controls.protectedInput.val('');
					}
				},
				'portInput invalid': function(e){
					if(!e.target.validity.valid){
						e.target.setCustomValidity(App.helpers._('port_number_format'));
						e.target.removeAttribute('pattern'); // turn off 
					}
				},
				'portInput input': function(e){
					e.target.setCustomValidity('');
					e.target.setAttribute('pattern', '^\\d+$');
				},
			}
		}).open();
	}
});
