define(['app', 'uicontrols', 'backbone', 'underscore' ], function(App, $AppUI, backbone, _){

// Attention:
// {Array} report.virus - can be empty
// {Int} stat.dev - number of device (filesytem)
// {Int} stat.mode - rights and protection mode
// {Int} stat.ino - number of node
// {Object} history[last]
// Threat to error group if 
// 	{Object} history[last].cure_report.error exist
var ThreatModel = backbone.Model.extend({
	// compare model with string
	isMatch: function(keyword){
		var 	str = '';

		for(var i = 0; i < this.textFileds.length; i++){
			str += this.dget(this.textFileds[i]) || '';
		}

		return str.indexOf(keyword) != -1;
	},
	dget: function(path){
		if(this.attributes[path]){
			return this.attributes[path];
		}else{
			var 	names = path.split("."),
					i = -1, 
					len = names.length, 
					ref = this.attributes;
					
			while(i++, i < len){
				ref = ref[names[i]];
				if(ref == undefined) break;
			}
			return ref;
		}
	},
	remove: function(){
		if(this.collection){ // Attention! Not fire any events
			this.collection.remove(this);
		}
	},
	parse: function(resp){
		if(resp.report){
			resp.vir_list = this._parseThreat(resp.report).map(function(obj){return obj.name;})
			resp.final_name = _.escape(resp.vir_list.join(', '));
			resp.origin = App.helpers._(resp.origin) || resp.origin;
		}
		return resp;
	},
	// TODO
	_parseThreat: function(report){
		var 	virList = [];

		if(Array.isArray(report.item) && report.item.length > 0){
			var 	i = report.item.length,
					res, j;

			while(i-- > 0){
				res = this._parseThreat(report.item[i]);
				j = res.length;

				while(j-- > 0){
					virList.push(res[j]);
				}
			}
		}else if(Array.isArray(report.virus) && report.virus.length > 0){
			virList = report.virus;
		}else if(report.heuristic_analysis){
			virList = [{
				name: App.helpers._('Suspicious_object')
			}];
		}

		return virList;
	},
	// for complete use fertch() method
	initialize: function(idConf){
		this.id = typeof(idConf) == 'object' ? idConf.entry_id + '@' + idConf.chunk_id : idConf;		
		this.url = (typeof(idConf) == 'object' ? '/api/10.2/qentry_info/' : '/api/10.2/threat_info/') + this.id;
	},
	sync: function(method, model, options){
    	// To get original responce: return Backbone.sync(method, model, options);
    	return App.xhr.get((this.url || options.url), null, function(d, r){
    		if(!r.ec && d) {
    			d.id = this.id || options.id;
    			if(options.clearBuffer) d.clearBuffer = options.clearBuffer;
    			delete this.url;
    			options.success && options.success(d, r);
    		}else{
    			options.error && options.error(d, r);
    		}
			    		
    	}.bind(this), 'json'); // with forse json parsing!
    },
	textFileds: ['final_name', 'origin', 'stat.gid.groupname', 'stat.uid.username', 'report.object', 'chunk_id', 'entry_id', 'restore_path'],
	threatModUrl:{
		cure: '/api/10.2/cure_threat/',
		delete: '/api/10.2/delete_threat/',
		ignore: '/api/10.2/ignore_threat/',
		quarantine: '/api/10.2/quarantine_threat/'
	},
	qentryModUrl:{
		cure: '/api/10.2/cure_qentry/',
		delete: '/api/10.2/delete_qentry/',
		restore: '/api/10.2/restore_qentry/',
	},
	// TODO move to collection
	// @param {String} mid - operation id
	// @param {Function} cb
	// @return {XmlHttpRequest} 
	modify: function(mid, cb, conf){
		this.collection.trigger('modifyStart', this, mid);
		var 	self = this,
				url = (this.get('threat_id') ? this.threatModUrl : this.qentryModUrl)[mid];

		// return url && App.xhr.post(url + this.id, null, function(d, r){
		// 	if(r.sessionComplete) return;
		// 	self.collection.trigger('modifyComplete', d, self, mid);
		// 	cb && cb(d, conf);
		// }, 'json');

		return url && App.xhr.post(url + this.id, null, _.debounce(function(d, r){
			if(r.sessionComplete) return;

			this.collection && this.collection.trigger('modifyComplete', d, this, mid);
			cb && cb(d, conf);
		}.bind(this)), 'json');
	},
});

var ThreatCollection = backbone.Collection.extend({
	model: ThreatModel,
	// DEPRICATED
	/*fetch: function(conf){
		var url = conf.threats ? '/api/10.2/threats': conf.qentry ? '/api/10.2/quarantine' : '';

		url && App.xhr.get(url, {}, function(d, r){
			if(Array.isArray(d)){
				// TODO add mark that item is bulk
				this.add(d);
			} else if(r.sessionComplete){
				return;	
			} // else another troubles
		}.bind(this), 'json');
	},*/
	// Start from last because if you add new model it would be fetched separetly at the end of models list
	resumeFetch: function(n){
		var 	n = n != undefined ? n : this.models.length,
				len = this.model.length,
				model;

		while(n-- > 0){
			model = this.models[n];
			if(model.hasOwnProperty('url')){ // never fetch model
				// renderBuffer n == 1 || (this.models.length - n)%10 == 0
				this._lastFetchRequest = model.fetch({
					clearBuffer: (n == 1 || (len - n) % 10 == 0), // at last an each 10 times
					success: function(){
						n > 0 && model.collection.resumeFetch(n);
						model = null;
					}
				});	
				break;
			}
		}
	},
	pauseFetch: function(){
		this._lastFetchRequest && this._lastFetchRequest.abort && this._lastFetchRequest.abort();
	},
	fetch: function(){
		var 	sourceList = ['/api/10.2/threats', '/api/10.2/quarantine'],
				count = sourceList.length;
				i = count;

		while(i-- > 0){
			App.xhr.get(sourceList[i], {}, function(d, r){
				if(Array.isArray(d)){
					// TODO add mark that item is bulk
					this.add(d); // get list of models
					if(--count == 0){ // last request and loop by this.models
						this.resumeFetch();
					}
				} else if(r.sessionComplete){
					return;	
				} // else another troubles
			}.bind(this), 'json');		
		}
	},
	clearAll: function(conf){
		if(!(conf && conf.silent)){
			for(var i = 0; i < this.models.length; i++){
				this.trigger('remove', this.models[i], this);
			}
		}
		this.reset();
	},
	select: function(modelId, value, silent){
		var model = this.get(modelId);

		model && model.set('selected', value);

		if(!silent){
			var 	i = this.models.length,
					majorGroup,
					majorType,
					type,
					current,
					count = 0,
					size = 0;

			while(i-- > 0){
				if(this.models[i].get('selected')){
					current = this.models[i];
					count++;
					size += current.dget('report.size');
					type = current.get('entry_id') ? 1 : 2; // 1 - qentry, 2 - tentry

					majorGroup = (majorGroup != undefined ? majorGroup == current.get('group_id') : true) && current.get('group_id');
					majorType = (majorType != undefined ? majorType == type : true) && type;
				}
			}		

			this.trigger('select', {
				current: count == 1 && current,
				count: count,
				size: size,
				majorGroup: majorGroup || (count == 0 ? 'blocked' : 'only_delete'),
				majorType: majorType,
			}, this);
		} 
	},
	getSelected: function(){
		var 	i = this.models.length,
				list = [];

		while(i-- > 0){
			if(this.models[i].get('selected')){
				list.push(this.models[i])
			}
		}

		return list;
	}
});

var ThreatPageModel = backbone.Model.extend({
	defaults: {
		active_group: '',
		select_all: false,
		query: '',
		page: 0, // UnusedNow
		active_count: 0,
		blocked_count: 0,
		quarantine_count: 0,
		error_count: 0,
		selected_items: [],
		// mod_errors: [], // function is turn off
	},
	// @param {String} key - model property
	increment: function(property, isReverse){
		this.set(property, this.get(property) + (isReverse ? -1 : 1));
	},
	getTotal: function(){
		//return this.attributes.active_count + this.attributes.blocked_count + this.attributes.error_count + this.attributes.quarantine_count;
		return this.attributes.active_count + this.attributes.blocked_count + this.attributes.quarantine_count;
	},
	alter: function(property, arg){
		var 	pos = property.indexOf(':'),
				prop,
				command;

		if(pos != -1){
			command = property.substr(pos + 1);
			property = property.substr(0, pos);
		}

		if((prop = this.get(property)) && Array.isArray(prop)){
			this._previousAttributes = {};
			this._previousAttributes[property] = prop.slice(); // store copy of previous value

			if(command == 'push'){
				prop.push(arg);
				arguments[2] != undefined && prop.push(arguments[2]);
			}else if(command == 'cleanout'){
				prop.length = 0;			
			}
			this.trigger(property + ':' + command, prop, this);
		}
	},
	isCurrentGroup: function(group){
		var 	activeGroup = this.get('active_group');
		return activeGroup ? activeGroup == group : true;	
	}
});
function getLast(list){
	return list[list.length - 1];
}
// ControlKit (Ckit) v8 2016/10/27
{
	// @param {String} arg1 - tagName
	// @param {String} className
	// @param {String} text - text content
	// Ovverided
	// @param {Element} arg1 - use node
	function Cr(arg1, className, text){
		if(!(this instanceof Cr)){
			return new Cr(arg1, className, text);
		}

		if(arg1 instanceof Element){
			this.el = arg1;
		}else{
			this.el = document.createElement(arg1);
			if(text) this.el.textContent = text;
			if(className) this.el.className = className;	
		}
		this.root = this.el;
	}
	Cr.prototype.append = function(arg1, className, text){
		var inst = new Cr(arg1, className, text);
		this.el.appendChild(inst.el);
		inst.root = this.root;
		return inst;
	};
	Cr.prototype.node = function(){
		return this.el;
	};
	Cr.prototype.parent = function(){
		if(this.el.parentNode){
			this.el = this.el.parentNode;	
		}
		
		return this;
	};
	// @param {String} arg1 - attribute name
	// @param {String} arg2 - attribute value
	// or
	// @param {Object} arg1 - collection of attributes
	// Attention: NS of property can be determined like 'xlink:href'
	Cr.prototype.attr = function(arg1, arg2){
		if(typeof(arg1) == 'string'){
			this.el.setAttribute(arg1, arg2);
		}else{
			for(var key in arg1){
				this.el.setAttribute(key, arg1[key]);
			}
		}
		return this;
	};
	// Create Alias for DOM element; Bind DOM Element with object property
	Cr.prototype.alias = function(name, collection){
		var co = collection || (this.co = Object.create(null));
		co[name] = this.el;
		return this;
	};
	Cr.prototype.use = function(cb, arg){
		return cb(this, arg), this;
	};
	Cr.prototype.prop = function(){
		for(var i = 0, m = arguments.length; i < m; i += 2){
			if(arguments[i] && arguments[i + 1]){
				this.el[arguments[i]] = arguments[i + 1];
			}
		}
		return this;
	};
	Cr.prototype.add = function(){
		for(var i = 0, len = arguments.length; i < len; i++){
			this.el.appendChild(arguments[i] instanceof this.constructor ? arguments[i].root : arguments[i]);
		}
		return this;
	};
	Cr.prototype.data = function(key, value){
		this.el.setAttribute('data-' + key, value);

		if(this.el.dataset){
			this.el.dataset[key] = value;	
		}
		return this;
	};

	Cr.fr = function(){
		var 	fr = document.createDocumentFragment();

		for(var i = 0, len = arguments.length; i < len; i++){
			if(arguments[i] instanceof Cr){
				fr.appendChild(arguments[i].root);
			}
		}
		return fr;
	};
	Cr.list = function(list, callback){
		var 	fr = document.createDocumentFragment(), 
				i, inst;

		for(i in list){
			inst = callback(list[i], i);
			fr.appendChild(inst instanceof Cr ? inst.root : inst);
		}
		return fr;
	}
}

	App.view.threats = App.view.base.extend({
		className: 'dwc_dcolumn-wrap __thr-page',
		_items: {},
		template: 
		'<div class="dwc_dcolumn-panel">' +
			'<div class="dwc_thr_panel-offset" data-co="right-panel"></div>' +
		'</div>' +
		'<div class="dwc_dcolumn-body">' +
			'<div class="dev_vertal">' +
				'<div class="dev_vertal-row">' +
					'<div class="dev_vertal-cell1">' +
						'<div class="dwc_thr_header-wrap">' +
							'<h2 class="dwc_thr_header" data-i18="threatening">' + App.helpers._('threatening') + '</h2>' +
							'<div class="dwc_thr_search-wrap">' + 
								'<input type="text" class="dwc_search-input dwc-form_base-input" data-co="search-input" />' +
								'<div class="dwc_search-btn" data-co="search-btn"></div>' +
							'</div>' +
							'<div class="dwc_thr_filters">' +
								'<label class="dwc_set_radio">' +
									'<input type="radio" name="group-cat" class="dwc_set_radio-checkbox" value="" checked>' + 
									'<span class="dwc_set_radio-label" data-i18="All" data-co="all-count" data-count="0">' + App.helpers._('All') + '</span>' +
								'</label>' +
								'<label class="dwc_set_radio">' +
									'<input type="radio" name="group-cat" class="dwc_set_radio-checkbox" value="active">' + 
									'<span class="dwc_set_radio-label" data-i18="Active" data-co="active-count" data-count="0">' + App.helpers._('Active') + '</span>' +
								'</label>' +
								'<label class="dwc_set_radio">' +
									'<input type="radio" name="group-cat" class="dwc_set_radio-checkbox" value="blocked">' + 
									'<span class="dwc_set_radio-label" data-i18="Blocked" data-co="blocked-count" data-count="0">' + App.helpers._('Blocked') + '</span>' +
								'</label>' +
								'<label class="dwc_set_radio">' +
									'<input type="radio" name="group-cat" class="dwc_set_radio-checkbox" value="quarantine">' + 
									'<span class="dwc_set_radio-label" data-i18="At_quarantine" data-co="quarantine-count" data-count="0">' + App.helpers._('At_quarantine') + '</span>' +
								'</label>' +
								'<label class="dwc_set_radio">' +
									'<input type="radio" name="group-cat" class="dwc_set_radio-checkbox" value="error">' + 
									'<span class="dwc_set_radio-label" data-i18="Errors" data-co="error-count" data-count="0">' + App.helpers._('Errors') + '</span>' +
								'</label>' +
							'</div>' +
						'</div>' +
						'<div class="dwc_thr_btn-wrap">' +
							'<button class="dwc_thr_btn-remove" data-co="delete-btn" title="delete" data-i18-attr="title:delete_threat"></button>' +
							'<button class="dwc_thr_btn-restore" data-co="restore-btn" title="restore" data-i18-attr="title:restore_threat"></button>' +
							'<a href="javascript:void(0)" class="dwc_thr_btn-more" data-co="more-btn" data-component="dropdown" data-list="" data-i18-attr="title:more_actions" title="" tabindex="0"></a>' + 
						'</div>' +
					'</div>' +
				'</div>' +
				'<div class="dev_vertal-row">' +
					'<div class="dev_vertal-cell2">' +
						'<div class="dwc_thr-scroll">' +
							'<div class="dwc_thr-colheader">' + 
								'<div class="dwc_thr-file_wrap clearfix dwc_thr-file_header" data-co="header">' +
									'<div class="dwc_thr-file_name" data-i18="File">' + App.helpers._('File') + '</div>' +
									'<div class="dwc_thr-file_owner" data-i18="Owner">' + App.helpers._('Owner') + '</div>' +
									'<div class="dwc_thr-file_component" data-i18="Component">' + App.helpers._('Component') + '</div>' +
									'<div class="dwc_thr-file_title" data-i18="check-popup_threat">' + App.helpers._('check-popup_threat') + '</div>' +
								'</div>' +
								'<label class="dwc_thr_check-all-label">' +
									'<input type="checkbox" class="dwc_thr-file_check-all" data-co="check-all"/>' +
									'<span class="dwc_thr_check-icn"></span>' +
								'</label>' +
							'</div>' +
							'<div data-co="items" data-group="" class="dwc_thr-collection"></div>' +
							'<pre class="dwc_thr-message" data-co="message-row" style="display:none;"></pre>' +
						'</div>' +
					'</div>' +
				'</div>' +
			'</div>' +
		'</div>' +
		'<div class="dwc_dcolumn-head" style="display:none;">' +
		'</div>',
		model: new ThreatPageModel(),
		// mode: 'default', // search
		// @param {Backbone.Model} model 
		renderItem: function(model, group){
			var		node = document.createElement('div'),
					ec = model.get('ec'),
					object = _.escape(model.dget('report.object')),
					fname = getLast(object.split('/')),
					userName = model.dget('stat.uid.username') || model.dget('stat.uid.uid');

			node.className = 'dwc_thr-file' + (ec ? ' __error' : '');
			node.dataset.group = group;
			node.setAttribute('data-group', group);
			ec && node.setAttribute('title', App.helpers._(ec));
			// if(who) user = (who.domain ? who.domain + '\\' : '') + who.user + (who.ip ? ' (' + who.ip + ')' : '');
			
			node.innerHTML = '<input type="checkbox" class="dwc_thr-file_check" data-id="' + _.escape(model.id) + '"/>' +
				'<div class="dwc_thr-file_wrap clearfix">' +
					'<span class="dwc_thr-checkbox"></span>' +
					'<div class="dwc_thr-file_name dwc_thr-type_icn">' + fname + '</div>' +
					'<div class="dwc_thr-file_owner">' + _.escape(userName) + '</div>' +
					'<div class="dwc_thr-file_component">' + _.escape(model.get('origin')) + '</div>' +
					(model.attributes && model.attributes.report && model.attributes.report.archive ? 
						'<div class="dwc_thr-file_title" data-i18="infected_archive">' + App.helpers._('infected_archive')  + '</div>' :
						'<div class="dwc_thr-file_title" title="' + model.get('final_name') + '">' + model.get('final_name') + '</div>'
					) +
				'</div>';
			this._items[model.id] = node;
			// old method:
			// this.controls.items.append(node);
			this._frameBuffer.appendChild(node);

			if(model.has('clearBuffer')){
				this.controls.items.append(this._frameBuffer);
			}
			this.sortItem();
		},
		sortItem: function(){
			var list = this.controls.items.context.childNodes,
				itemsArr = [];

			for(var i in list) {
				if (list[i].nodeType == 1) {
			 		itemsArr.push(list[i]);
			 	}
			}

			itemsArr.sort(function(a, b) {
			 	return a.firstChild.getAttribute('data-id') == b.firstChild.getAttribute('data-id') ? 0 : (a.firstChild.getAttribute('data-id') > b.firstChild.getAttribute('data-id') ? 1 : -1); 
			});

			this.controls.items.append(itemsArr);
		},
		_frameBuffer: document.createDocumentFragment(),
		formatTime: App.helpers.timeFormatter(App.helpers._('DateStr')), // '%h:%m - %D.%M.%Y'
		renderThreatDetails: function(model){
			var 	name = model.get('final_name'),
					groupClass = 'dwc_thr_fname ',
					ec = model.escape('ec');
			
			switch(model.get('group_id')){
				case 'quarantine': groupClass += '__quarantine'; break;
				case 'blocked': groupClass += '__blocked'; break;
				case 'error': groupClass += '__error'; break;
			}

			var $threat;

			if(model.attributes && model.attributes.report && model.attributes.report.archive){
				$threat = Cr('div', 'dwc_thr_group-value').
					append('a', 'dwc_thr_discoveredlist-title', App.helpers._('infected_archive')).
					prop('href', 'javascript:void(0)').
					data('i18', 'infected_archive').
					prop('onclick', function(){
						new App.ShowThreatsPopup([model.get('report')]);
					});
			}else{
				$threat = Cr('div', 'dwc_thr_group-value', model.get('final_name'));
			}
			var 	user = model.dget('stat.uid.username') || model.dget('stat.uid.uid'),
					group = model.dget('stat.gid.groupname') || model.dget('stat.gid.gid'),
					who = model.get('who');

			who = who ? (who.domain ? who.domain + '\\' : '') + who.user + (who.ip ? ' (' + who.ip + ')' : '') : false;
			this.controls.rightPanel.empty().append(Cr.fr(
				Cr('div', groupClass, model.dget('report.object')),
				ec && Cr('div', 'dwc_thr_group').append('div', 'dwc_thr_group-value dwc_thr_error', App.helpers._(ec)).data('i18', ec),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Threat')).data('i18', 'Threat'),
					$threat
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Size')).data('i18', 'Size'),
					Cr('div', 'dwc_thr_group-value', App.helpers.formatBytes(model.dget('report.size'), ['vol_B', 'vol_Kb', 'vol_Mb', 'vol_Gb', 'vol_Tb'].map(function(k){return ' ' + App.helpers._(k);})))
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Discovered')),
					Cr('div', 'dwc_thr_group-value', model.get('origin'))
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Date_of_discovery')).data('i18', 'Date_of_discovery'),
					Cr('div', 'dwc_thr_group-value', this.formatTime(model.dget('stat.ctime')))
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Last_change')).data('i18', 'Last_change'),
					Cr('div', 'dwc_thr_group-value', this.formatTime(model.dget('stat.mtime')))
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Object_owner')).data('i18', 'Object_owner'),
					Cr('div', 'dwc_thr_group-value', user)
				),
				who && Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Remote_user')).data('i18', 'Remote_user'),
					Cr('div', 'dwc_thr_group-value', who)
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('User_group')).data('i18', 'User_group'),
					Cr('div', 'dwc_thr_group-value', group)
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('threat-id')).data('i18', 'threat-id'),
					Cr('div', 'dwc_thr_group-value', model.id)
				),
				Cr('div', 'dwc_thr_group').add(
					Cr('h4', 'dwc_thr_group-title', App.helpers._('Path')).data('i18', 'Path'),
					Cr('div', 'dwc_thr_group-value', model.dget('report.object'))
				)
			));
		},
		renderThreatListData: function(count, size){
			this.controls.rightPanel[0].innerHTML = '<div class="dwc_thr_group">' +
				'<h4 class="dwc_thr_group-title"><span data-i18="Selected_objects">' + App.helpers._('Selected_objects') + '</span></h4>' +
				'<div class="dwc_thr_group-value">' + count + '</div>' +
			'</div>' +
			'<div class="dwc_thr_group">' +
				'<h4 class="dwc_thr_group-title"><span data-i18="Size">' + App.helpers._('Size') + '</span></h4>' +
				'<div class="dwc_thr_group-value">' + App.helpers.formatBytes(size, ['vol_B', 'vol_Kb', 'vol_Mb', 'vol_Gb', 'vol_Tb'].map(function(k){return ' ' + App.helpers._(k);})) + '</div>' +
			'</div>';
		},
		createThreatPopup: function(conf){
			var 	fileList = conf.list,
					_xhr,
					self = this;

			new $AppUI.BasePopupView({
					title: conf.popupTitle,
					class: 'dec_wt-popup',
					content: 
					'<form class="dwc-form" data-co="form">' +
						'<div class="dwc-form_offset1">' + conf.popupMessage + '</div>' +
						'<div class="dwc-form_btn-wrap">' +
							'<button class="dwc_btn" type="submit">' + App.helpers._('Yes') + '</button>' +
							'<button class="dwc_btn" data-co="cancel">' + App.helpers._('No') + '</button>' +
						'</div>' +
					'</form>' +
					'',
					onopen: function(view){
						view._bindByRole(view.controls.body[0]);
						view.controls.cancel.on('click', function(){view.close();});
						view.controls.form.on('submit', function(e){
							e.preventDefault();
							for(var i = 0; i < conf.list.length; i++){
								// 1. nodes[i] would be passed at conf.action as argument
								// 2. DOM manipulation with main threat list would be at modify event handler

								conf.list[i].modify(conf.method, conf.action);
							}
							self.model.set('select-all', false); // TODO hide check from select all
							view.close();
						});
					},
					onclose: function(view){
						fileList.length = 0;
					}
				}).open();
		},
		resetAllCheckBox: function(){
			var 	$checkList = this.controls.items.find('.dwc_thr-file_check'),
					i = $checkList.length;

			while(i-- > 0) {
				$checkList[i].checked = false;
			}
			this.threatCollection.trigger('selectAll', false, this.threatCollection, true);
		},
		events:{
			'dropdownselected [data-co="more-btn"]': function(e){
				var 	id = e.target.dataset.id,
						selected = this.threatCollection.getSelected(),
						itemModel,
						i = selected.length;

				if(
					id == 'cure' ||
					id == 'quarantine' ||
					id == 'ignore'
				){
					while(i-- > 0){
						itemModel = selected[i];
						if(!itemModel || itemModel.get('modify')) continue; // skip removed items and items with runned and not complited operations
						itemModel.modify(id);	
					}
				}
				this.resetAllCheckBox();
			},
			'change .dwc_thr_filters': function(e){ // filter switcher
				this.model.set('active_group', e.target.value);
			},
			// check All controls #1
			'change .dwc_thr-file_check-all': function(e){
				// @params: new value, collection, without model change
				this.threatCollection.trigger('selectAll', e.target.checked, this.threatCollection, false);
			},
			'focus .dwc_thr-file_check-all': function(){
				this.controls.header.addClass('__focus');
			},
			'blur .dwc_thr-file_check-all': function(){
				this.controls.header.removeClass('__focus');
			},
			'click [data-co="delete-btn"]': function(e){
				var 	selected = this.threatCollection.getSelected(),
						i = selected.length;

				if(i == 0) return;

				var conf = {
					list: [],
					popupTitle: App.helpers._('Removing'),
					popupMessage: '',
					method: 'delete',
				};

				while(i-- > 0){
					if(!selected[i].get('modify')){
						conf.list.push(selected[i]);	
					}
				}
				conf.popupMessage = App.helpers._(conf.list.length == 1 ? 'Do_you_want_remove_this_file' : conf.list.length > 1 ? 'Do_you_want_remove_this_files' : 'No_files_selected').replace('%n', conf.list.length);
				this.createThreatPopup(conf);
				this.resetAllCheckBox();
			},
			'click [data-co="restore-btn"]': function(e){
				var 	selected = this.threatCollection.getSelected(),
						i = selected.length;

				if(i == 0) return;

				var conf = {
					list: [],
					popupTitle: App.helpers._('Restore_from_quarantine'),
					popupMessage: '',
					method: 'restore',
				};
				
				while(i-- > 0){
					if(!selected[i].get('modify') && selected[i].get('chunk_id')){
						conf.list.push(selected[i]);	
					}
				}
				conf.popupMessage = App.helpers._(conf.list.length == 1 ? 'Do_you_want_restore_this_file' : conf.list.length > 1 ? 'Do_you_want_restore_this_files' : 'No_files_selected').replace('%n', conf.list.length);
				this.createThreatPopup(conf);
				this.resetAllCheckBox();
			},
			// click on checkbox -> add current item at list or remove from list
			'click .dwc_thr-checkbox': function(e){
				e.stopPropagation();
				var 	$checkbox = e.target.parentNode.previousSibling,
						newState = !$checkbox.checked;

				// @params: new value, collection, without model change
				this.threatCollection.trigger('selectAll', false, this.threatCollection, true);
				this.threatCollection.select($checkbox.dataset.id, newState);
				$checkbox.checked = newState;
			},
			// click by list item -> select only current item
			'click .dwc_thr-file': function(e){
				// Attention: this.currentSelected is used for showing threat data at sidebar, not last focused item at list.
				// Use this._lastFocused for this
				var 	$target = e.currentTarget,
						$checkbox = $target.querySelector('.dwc_thr-file_check'),
						currSelectedId = $checkbox.dataset.id,
						$prev = this._items[this._lastFocused],
						newState = true;

				if(
					(e.shiftKey || e.metaKey) && 
					$prev && 
					this.model.isCurrentGroup($prev.dataset.group) && !$prev.classList.contains('__hbs')
				){ // when press [SHIFT]
					var 	prevTop = $prev.offsetTop,
							currTop = this._items[currSelectedId] && this._items[currSelectedId].offsetTop,
							direction = prevTop != undefined && (prevTop - currTop),
							nextMethod = direction && (direction > 0 ? 'nextElementSibling' : 'previousElementSibling'),
							$item,
							$itemCheckbox;

					if($item = e.currentTarget){
						while($item && $item != this._items[this._lastFocused]){
							$itemCheckbox = $item.querySelector('.dwc_thr-file_check')
							
							// TODO skip HBS and another group
							if(this.model.isCurrentGroup($item.dataset.group)  && !$item.classList.contains('__hbs') && $itemCheckbox){
								$itemCheckbox.checked = newState;
								this.threatCollection.select($itemCheckbox.dataset.id, newState, true);
							}
							$item = $item[nextMethod];
						}
					}
					this.threatCollection.trigger('selectAll', false, this.threatCollection, true);
				}else if(e.ctrlKey){
					newState = !$checkbox.checked;
					this.threatCollection.trigger('selectAll', false, this.threatCollection, true);
				}else{
					// @params: new value, collection, without model change
					this.threatCollection.trigger('selectAll', false, this.threatCollection, false);
				}
				this.threatCollection.select(currSelectedId, newState);
				$checkbox.checked = newState;
				this._lastFocused = currSelectedId;
			},
			// Turn off selection by [Shift] & CLICK at IE
			'selectstart .dwc_thr-collection': function(e){ 
				e.preventDefault();
				return false;
			},
			'input .dwc_search-input': function(e){
				this.search(e.target.value);
			},
			'click .dwc_clear-btn': function(e){
				this.controls.searchInput.val('');
				this.search('');	
			},
		},
		search: _.debounce(function(val){
			this.model.set('query', val);
		}, 200),
		configureActionButtons: function(group, type){
			switch(group){
				case 'blocked': // locked all
					this.controls.restoreBtn.prop('disabled', true);

					if(type){
						var listStr = type == 1 ? 'cure;' + App.helpers._('cure_threat') : 'quarantine;' + App.helpers._('quarantine_threat') + ';cure;' + App.helpers._('cure_threat') + ';ignore;' + App.helpers._('ignore_threat');

						this.controls.deleteBtn.prop('disabled', false);
						this.controls.moreBtn.removeAttr('disabled');
						this.controls.moreBtn.data('list', listStr).attr('data-list', listStr);
					}else{
						this.controls.deleteBtn.prop('disabled', true);
						this.controls.moreBtn.attr('disabled', 'disabled');						
					}

					break;
				case 'quarantine': // allow: удалить, лечить, восстановить; disallow: поместить в карантин и игнорировать
					this.controls.deleteBtn.prop('disabled', false);
					this.controls.restoreBtn.prop('disabled', false);
					this.controls.moreBtn.removeAttr('disabled');
					var listStr = 'cure;' + App.helpers._('cure_threat');
					this.controls.moreBtn.data('list', listStr).attr('data-list', listStr);
					break;
				case 'only_delete': // for mixed group
					this.controls.deleteBtn.prop('disabled', false);
					this.controls.restoreBtn.prop('disabled', true);
					this.controls.moreBtn.attr('disabled', 'disabled');
					break;
				case 'active': // disallow: восстаноить из карантина
				case 'error': 
				case '': // all
					var 	listStr;

					if(type == 1 && group == 'error'){ // for qentry with error status
						this.controls.restoreBtn.prop('disabled', false);
						listStr = 'cure;' + App.helpers._('cure_threat');
					}else{
						this.controls.restoreBtn.prop('disabled', true);
						listStr = 'quarantine;' + App.helpers._('quarantine_threat') + ';cure;' + App.helpers._('cure_threat') + ';ignore;' + App.helpers._('ignore_threat');
					}

					this.controls.deleteBtn.prop('disabled', false);
					this.controls.moreBtn.removeAttr('disabled');
					this.controls.moreBtn.data('list', listStr).attr('data-list', listStr);
					break;
			}
		},
		checkAllCount: function() {
			if(this.controls.allCount.attr('data-count') == 0) {
			 	this.controls.messageRow.text(App.helpers._('no_threats')).show();
			} else {
				this.controls.messageRow.hide();
			}
		},
		initialize: function(conf){
			App.view.base.prototype.initialize.call(this);
			this.threatCollection = new ThreatCollection();
			this.checkAllCount();
			this.listenTo(this.threatCollection, 'select', function(data, collection){
				if(data.current){ // render currentSelected
					this.renderThreatDetails(data.current);
				}else if(data.count > 1){
					this.renderThreatListData(data.count, data.size);
				}else{
					this.controls.rightPanel.empty();
				}

				this.currentSelected = data.current; // USE to rewrite
				this.configureActionButtons(data.majorGroup, data.majorType);	
			});
			// @param {Bool} silent - without model change
			this.listenTo(this.threatCollection, 'selectAll', function(checked, collection, silent){
				this.controls.checkAll.prop('checked', checked);
				
				if(!silent){
					var 	$checkList = this.controls.items.find('.dwc_thr-file_check'),
							group = this.controls.items.data('group'),
							i = $checkList.length;

					if(group == 'error'){
						while(i-- > 0){
							if(
								$checkList[i].parentNode.classList.contains('__hbs') || // skip item if it was hidden by searching
								!$checkList[i].parentNode.classList.contains('__error')
							){
								collection.select($checkList[i].dataset.id, false); 
								continue;
							}
							$checkList[i].checked = checked;
							collection.select($checkList[i].dataset.id, checked);
						}
					}else{
						while(i-- > 0){
							if(
								$checkList[i].parentNode.classList.contains('__hbs') || // skip item if it was hidden by searching
								(group && ($checkList[i].parentNode.dataset.group != group))
							){
								collection.select($checkList[i].dataset.id, false); 
								continue;
							}
							$checkList[i].checked = checked;
							collection.select($checkList[i].dataset.id, checked);
						}						
					}
				}
			});
			
			App.helpers.localizeAttr(this.el);
			this.controls.rightPanel.empty();
			this.configureActionButtons('blocked');
			
			// On group change:
			this.listenTo(this.model, 'change:active_group', function(model, group){
				this.resetAllCheckBox();
				this.controls.items.attr('data-group', group).data('group', group);
				this.model.set('query', '');
				this.controls.searchInput.val('');
				// @params: new value, collection, without model change
				this.threatCollection.trigger('selectAll', false, this.threatCollection, false);
			});

			// function is turnOf
			/*this.listenTo(this.model, 'mod_errors:push', function(errors, model){
				if(!this.modErrorNotify){
					this.modErrorNotify = new $c.BasePopupView({
						// title: '<strong>' + _.escape(title) + '</strong>',
						class: 'dec_wt-popup',
						content: 
						'<form class="dwc-form __break" data-co="form">' +
							'<div class="dwc-form_offset1 " data-co="list"></div>' +
							'<div class="dwc-form_btn-wrap">' +
								'<button class="dwc_btn" data-co="submit">' + App.helpers._('Close') + '</button>' +
							'</div>' +
						'</form>' +
						'',
						onopen: function(view){
							view.controls.form.on('submit', function(e){
								e.stopPropagation();
								e.preventDefault();
								view.close();
							});
						},
						onclose: function(view){
							this.model.alter('mod_errors:cleanout');
							this.modErrorNotify = null;
						}.bind(this)
					});
					this.modErrorNotify.open();
				}
				// set items
				this.modErrorNotify.controls.popupTitle.text(App.helpers._(errors.length > 1 ? 'action_with_errors' : 'action_with_error'));
				var 	$frag = document.createDocumentFragment(),
						$node, $span;

				for(var i = 0; i < errors.length; i += 2){
					$node = document.createElement('div');
					$span = document.createElement('span');
					$span.className = 'dwc_thr_error';
					$node.appendChild(document.createTextNode(errors[i] + ' - '));
					$span.appendChild(document.createTextNode(App.helpers._(errors[i + 1])));
					$node.appendChild($span);
					$frag.appendChild($node);
				}

				this.modErrorNotify.controls.list.empty().append($frag);
			});*/

			this.listenTo(this.model, 'change:query', function(model, query){
				var 	self = this,
						group = model.get('active_group'),
						count = 0;
				if(query){
					this.controls.searchBtn.addClass('dwc_clear-btn');
					this.controls.items.find('[data-id]').each(function(index, $input){
						if(group && $input.parentNode.dataset.group != group && !($input.parentNode.classList.contains('__error'))){ // skip items from another groups
							return;
						}

						var 	id = $input.dataset.id,
								itemModel = this.threatCollection.get(id);

						$input.checked = false;

						if(itemModel && itemModel.isMatch(query)){
							count++;
							$input.parentNode.classList.remove('__hbs'); // "Hide By Search"
						}else{
							$input.parentNode.classList.add('__hbs');
						}
					}.bind(this));

					if(!count){
						self.controls.messageRow.text(App.helpers._('No_records_satisfying_the_request')).show();
					}else{
						self.checkAllCount();
					}
				}else{
					this.controls.searchBtn.removeClass('dwc_clear-btn');
					this.controls.items.children().removeClass('__hbs'); // "Hide By Search"
					this.checkAllCount();
				}
				// @params: new value, collection, without model change
				this.threatCollection.trigger('selectAll', false, this.threatCollection, false);
			});
			this.listenTo(this.model, 'change:active_count', function(model, count){
				this.controls.allCount.attr('data-count', this.model.getTotal());
				this.checkAllCount();
				this.controls.activeCount.attr('data-count', count);
			});
			this.listenTo(this.model, 'change:quarantine_count', function(model, count){
				this.controls.allCount.attr('data-count', this.model.getTotal());
				this.checkAllCount();
				this.controls.quarantineCount.attr('data-count', count);
			});
			this.listenTo(this.model, 'change:blocked_count', function(model, count){
				this.controls.allCount.attr('data-count', this.model.getTotal());
				this.checkAllCount();
				this.controls.blockedCount.attr('data-count',count);
			});
			this.listenTo(this.model, 'change:error_count', function(model, count){
				this.controls.allCount.attr('data-count', this.model.getTotal());
				this.checkAllCount();
				this.controls.errorCount.attr('data-count', count);
			});
			this.listenTo(this.threatCollection, 'modifyStart', function(model, mid){
				model.set('modify', true);
				this.controls.items.find('[data-id="' + model.id + '"]').attr('action', 'progress');
			});
			this.listenTo(this.threatCollection, 'modifyComplete', function(d, model, mid){
				var 	$threatBox = this.controls.items.find('[data-id="' + model.id + '"]'),
						$threatRow = $threatBox.parent(),
						threatGroup = $threatRow.data('group');

				model.set('modify', false);
				$threatRow.removeAttr('title');

				if(d && d.error && d.error.code){ // if fail
					//// function is turn Off
					// var path = model.get('report').object;
					// if(model.get('ec') && path){
					// 	this.model.alter('mod_errors:push', path, d.error.code);
					// }
					
					$threatRow.hasClass('__error') || this.model.increment('error_count');
					$threatBox.attr('action', 'error');
					$threatRow.addClass('__error');
					model.get('selected') && $threatBox.prop('checked', true);
					d.error.code && $threatRow.attr('title', App.helpers._(d.error.code));
					model.set('ec', d.error.code);

					if(model == this.currentSelected){
						this.renderThreatDetails(model);

					}
				}else{ // operation success
					$threatBox.attr('action', '');

					if(mid == 'delete' || mid == 'restore' || mid == 'ignore' || mid == 'cure' || mid == 'quarantine'){ // success delete
						$threatRow.remove();
						model.remove();
					}
				}
			});
			// Prepare threat when it fetched:
			this.listenTo(this.threatCollection, 'change:id', function(model){
				var 	group,
						history = model.get('history');

				if(Array.isArray(history)){
					history = history[history.length - 1]; // Attention: change {Array} -> {Object}

					if(history && (history.result != 'EC_OK' || history.cure_report && history.cure_report.error)){
						this.model.increment('error_count');
						model.set('ec', history.result);
					}
				}

				if(model.get('threat_id')){
					if(model.get('locked_by')){
						this.model.increment('blocked_count');
						group = 'blocked';
					}else{
						this.model.increment('active_count');
						group = 'active';
					}
				}else if(model.get('entry_id')){ //  this is quarantine item... 
					if(model.get('locked_by')){
						this.model.increment('blocked_count');
						group = 'blocked';
					}else{
						this.model.increment('quarantine_count');
						group = 'quarantine';
					}
				}

				model.set('group_id', group);
				this.renderItem(model, group);
			});
			this.on('remove', function(){
				this.threatCollection.reset();
			});

			// clear page
			this.listenTo(this.threatCollection, 'reset', function(){
				this.model.set('blocked_count', 0);
				this.model.set('error_count', 0);
				this.model.set('active_count', 0);
				this.model.set('quarantine_count', 0);
				this.model.set('active_group', '');
				this.controls.items.empty();

				// at testing:
				if(this.threatCollection._lastFetchRequest && this.threatCollection._lastFetchRequest.abort){
					this.threatCollection._lastFetchRequest.abort();
				}
				this._items = {};
			});

			// TODO test
			this.listenTo(this.threatCollection, 'remove', function(model, collection){
				var 	group = model.get('group_id'),
						//$threatBox = this.controls.items.find('[data-id="' + model.id + '"]');
						$threat = this._items[model.id];

				if(model.has('ec')){
					this.model.increment('error_count', true);
				}

				switch(group){
					case 'quarantine':
						this.model.increment('quarantine_count', true);
						break;
					case 'active':
						this.model.increment('active_count', true);
						break;
					case 'blocked':
						this.model.increment('blocked_count', true);
						break;
				}

				if(model.get('selected')){
					collection.select();
				}

				// $threatBox.parent().remove();
				$threat.remove();
				model = null;
			});

			this.threatCollection.reset();
			this.threatCollection.fetch();

			// fix table header
			setTimeout(function(){
				var 	$tableWrap = this.controls.items[0],
						$headerRow = this.controls.header[0],
						barWidth = $tableWrap.offsetWidth - $tableWrap.scrollWidth; // detect width of scroll bar:
				
				$headerRow.style.paddingRight = barWidth + 'px'; // set relative padding:
			}.bind(this), 0);

			this.listenTo(App.bus, 'refresh', function(e){
				var 	diff, i, action;

				// listen to quarantine events
				if(e.event == 'quarantine'){
					for(i = 0; i < e.event_data.length; i++){
						action = e.event_data[i].action;
						
						if(action == 'REMOVED' || action == 'MODIFIED'){ // remove entry
							var 	id = e.event_data[i].entry_id + '@' + e.event_data[i].chunk_id,
									model4remove = this.threatCollection.get(id);

							model4remove && model4remove.remove();
						}
						
						if(action == 'ADDED' || action == 'MODIFIED'){ // add new entry
							this.threatCollection.add(e.event_data[i]).fetch({
								clearBuffer: true
							});
						}
					}
				}else if(e.event == 'threats'){
					diff = e.event_data && (e.event_data.diff || e.event_data);

					for(i = 0; i < diff.length; i++){
						action = diff[i].action;
						
						if(action == 'REMOVED' || action == 'MODIFIED'){ // remove entry
							var 	model4remove = this.threatCollection.get(diff[i].id);

							model4remove && model4remove.remove();
						}

						if(action == 'ADDED' || action == 'MODIFIED'){ // add new entry
							this.threatCollection.add(diff[i].id).fetch({
								clearBuffer: true
							});
						}
					}
				}
			});

			this.controls.moreBtn.onblur = function(e){
				e.target.classList.remove('__opened');
				e.target.click();
			};
		},
	});

	return App.view.threats;
});	